@@ -18,14 +18,14 @@ from tensorflow.keras.initializers import (
     truncated_normal as tf_truncated_normal
 )
 from .base import Base, TfMixin
+from ..data.data_generator import DataGenFeat
 from ..evaluation.evaluate import EvalMixin
 from ..utils.tf_ops import (
     reg_config,
     dropout_config,
-    dense_nn,
-    lr_decay_config
+    lr_decay_config,
+    multi_sparse_combine_embedding
 )
-from ..data.data_generator import DataGenFeat
 from ..utils.sampling import NegativeSampling
 from ..utils.misc import count_params
 from ..feature import (
@@ -61,6 +61,7 @@ class FM(Base, TfMixin, EvalMixin):
             use_bn=True,
             dropout_rate=None,
             batch_sampling=False,
+            multi_sparse_combiner="sqrtn",
             seed=42,
             lower_upper_bound=None,
             tf_sess_config=None
@@ -90,6 +91,10 @@ class FM(Base, TfMixin, EvalMixin):
         if self.sparse:
             self.sparse_feature_size = self._sparse_feat_size(data_info)
             self.sparse_field_size = self._sparse_field_size(data_info)
+            self.multi_sparse_combiner = self._check_multi_sparse(
+                data_info, multi_sparse_combiner)
+            self.true_sparse_field_size = self._true_sparse_field_size(
+                data_info, self.sparse_field_size, self.multi_sparse_combiner)
         if self.dense:
             self.dense_field_size = self._dense_field_size(data_info)
         self.all_args = locals()
@@ -130,6 +135,7 @@ class FM(Base, TfMixin, EvalMixin):
                                         units=1,
                                         activation=tf.nn.elu)
         self.output = tf.squeeze(tf.add(linear_term, pairwise_term))
+        count_params()
 
     def _build_user_item(self):
         self.user_indices = tf.placeholder(tf.int32, shape=[None])
@@ -187,10 +193,20 @@ class FM(Base, TfMixin, EvalMixin):
             initializer=tf_truncated_normal(0.0, 0.03),
             regularizer=self.reg)
 
-        linear_sparse_embed = tf.nn.embedding_lookup(    # B * F1
-            linear_sparse_feat, self.sparse_indices)
-        pairwise_sparse_embed = tf.nn.embedding_lookup(  # B * F1 * K
-            pairwise_sparse_feat, self.sparse_indices)
+        if (self.data_info.multi_sparse_combine_info
+                and self.multi_sparse_combiner in ("sum", "mean", "sqrtn")):
+            linear_sparse_embed = multi_sparse_combine_embedding(
+                self.data_info, linear_sparse_feat, self.sparse_indices,
+                self.multi_sparse_combiner, 1)
+            pairwise_sparse_embed = multi_sparse_combine_embedding(
+                self.data_info, pairwise_sparse_feat, self.sparse_indices,
+                self.multi_sparse_combiner, self.embed_size)
+        else:
+            linear_sparse_embed = tf.nn.embedding_lookup(    # B * F1
+                linear_sparse_feat, self.sparse_indices)
+            pairwise_sparse_embed = tf.nn.embedding_lookup(  # B * F1 * K
+                pairwise_sparse_feat, self.sparse_indices)
+
         self.linear_embed.append(linear_sparse_embed)
         self.pairwise_embed.append(pairwise_sparse_embed)
 
